/*
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Id: espda_dfs.h 161574 2014-05-23 11:59:12Z gcupani $
 * $Author: gcupani $
 * $Date: 2014-05-23 13:59:12 +0200 (Fri, 23 May 2014) $
 * $Revision: 161574 $
 */

#ifndef ESPDA_DFS_H
#define ESPDA_DFS_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_dfs_call(A, B) \
            if (A != 0) { \
                int v_level_temp = v_level; \
                v_level = A; \
                B; \
                v_level = v_level_temp; \
            } \
            else { \
                B; \
            } \

#define espda_dfs_ensure(A)                                         \
            if (A != CPL_ERROR_NONE) return cpl_error_get_code();   \

/*
#define espda_dfs_ensure(A)                                         \
           cpl_ensure_code(A == CPL_ERROR_NONE, cpl_error_get_code());   \
*/

#define INV_INT        	-9999             // Invalid integer value
#define INV_DOUBLE     	-9999.0           // Invalid double  value
#define INV_STRING     	"VOID"            // Invalid string  value

#define LEN_STRING     	10000             // Length of string variables

#define MODEL_SIZE     	101               // Size of the instrument model
#define MODEL_RESOL    	60000             // Resolution of the instr. model
#define MODEL_KAPPA    	5.0               // # of st. dev. to cut the model

#define ECHARGE2_EMASS 	2.817940283e-8    // Ratio between electron charge
                       	                  // squared and electron mass
#define ERADIUS        	2.8179403267e-15  // Electron radius [m]
#define LIGHTSPEED     	299792.458        // Speed of light  [km/s]
#define LIGHTSPEED1000 	299792458         // Speed of light  [m/s]
#define FWHM_TO_STDEV  	2.354820045       // FWHM-to-standard deviation ratio

#define TAU_NORM       	0.0028            // PL normalization of the opt. d.
#define TAU_INDEX      	3.45              // PL index         of the opt. d.

#define WAVEL_START    	200.00            // Start wavelength (380.0) [nm]
#define WAVEL_END      	10000.00          // End wavelength   (780.0) [nm]
#define WAVEL_STEP     	0.01              // Wavelength step  [nm]

#define VEL_STEP       	5.0               // Velocity step    [km/s]
#define VEL_PROX       	5000.0            // Range of the prox. region [km/s]

#define COLDEN_1       	12.0              // Threshold column density values
#define COLDEN_2       	14.0              // to define the effective opt. d.
#define COLDEN_3       	18.0
#define COLDEN_4       	19.0
#define COLDEN_INDEX   	-1.65              // PL index of the col. density PDF

#define WAVEL_LY_A      121.5670000       // Lyman alpha wavelength
#define WAVEL_NV_1242   124.2804000       // N  V  1242  wavelength
#define WAVEL_CIV_1550  155.0781000       // C  IV 1550  wavelength
#define WAVEL_MGII_2803 280.3531500       // Mg II 2803  wavelength

#define PAR_MAX         20                // Maximum number of parameters
#define TAG_MAX         20                // Maximum number of tags
#define FRAME_MAX       50                // Maximum number of frames
#define GRP_MAX         1000              // Maximum number of line groups

#define TRANSF_MODE     "T"               // Transform mode for Voigt parameters
                                          // T: tangent, A: arc sine
#define ZEM_UNC         1.0e3             // Uncertainty on em. redshift [km/s]

/*
 * DO.CATG keywords
 */

// 1D frames
#define S1D              "S1D"

// 2D frames
#define S2D              "S2D"
#define S2D_SKYSUB       "S2D_SKYSUB"
#define BLAZE_2D         "BLAZE_FIBER"
#define FLUX_2D          "SPEC_FLUX_2D"
#define FLUXERR_2D       "SPEC_FLUXERR_2D"
#define WAVEL_2D         "WAVE_MATRIX_FIBER"
#define FLUX_2D_DRS      "SPEC_FLUX_2D_DRS"
#define FLUXERR_2D_DRS   "SPEC_FLUXERR_2D_DRS"
#define WAVEL_2D_DRS     "SPEC_WAVEL_2D_DRS"

// Resolution frame
#define RES_MAP          "RES_MAP_FIBER"

// Array frames
#define SPEC_ARRAY       "SPEC_ARRAY"


/*
 * PRO.CATG keywords
 */

// Spectra (SPEC)
#define SPEC             "SPEC"
#define MERG_SPEC        "MERG_SPEC"
#define MERG_SPEC_MASK   "MERG_SPEC_MASK"
#define COADD_SPEC       "COADD_SPEC"
#define CHECK_SPEC       "CHECK_SPEC"
#define ORD_SPEC         "ORD_SPEC"
#define ORD_SPEC_MASK    "ORD_SPEC_MASK"
#define ORD_SPEC_NORM    "ORD_SPEC_NORM"
#define EQ_PROF          "EQ_PROF"
#define EQ_SPEC          "EQ_SPEC"
#define EQ_SPEC_MASK     "EQ_SPEC_MASK"
#define EQ_SPEC_NORM     "EQ_SPEC_NORM"
#define EQ_SPEC_GUESS    "EQ_SPEC_GUESS"
#define EQ_SPEC_VOIGT    "EQ_SPEC_VOIGT"
#define REBIN_PROF       "REBIN_PROF"
#define REBIN_SPEC       "REBIN_SPEC"
#define REBIN_SPEC_MASK  "REBIN_SPEC_MASK"
#define REBIN_SPEC_NORM  "REBIN_SPEC_NORM"
#define REBIN_SPEC_GUESS "REBIN_SPEC_GUESS"
#define REBIN_SPEC_REM   "REBIN_SPEC_REM"
#define REBIN_SPEC_VOIGT "REBIN_SPEC_VOIGT"
#define NORD_SPEC        "NORD_SPEC"
#define NMERG_SPEC       "NMERG_SPEC"
#define SMERG_SPEC       "SMERG_SPEC"
#define RMERG_SPEC       "RMERG_SPEC"
#define CCF_SPEC         "CCF_SPEC"
#define VOIGT_SPEC       "VOIGT_SPEC"
#define SYNTH_SPEC       "SYNTH_SPEC"

// Full spectra (FSPEC)
#define FSPEC_ORIG       "FSPEC_ORIG"     // Original from reduction
#define FSPEC_PRE        "FSPEC_PRE"      // Prepared by coaddition
#define FSPEC            "FSPEC"          // Standard
#define FSPEC_CONT       "FSPEC_CONT"     // Continuum-fitted
#define FSPEC_REM        "FSPEC_REM"      // With lines removed
#define FSPEC_GUESS      "FSPEC_GUESS"    // With Voigt-guessed lines
#define FSPEC_VOIGT      "FSPEC_VOIGT"    // With Voigt-fitted  lines

// Instrument profile for full spectra (FPROF)
#define FPROF            "FPROF"

// Rebinned spectra (RSPEC)
#define RSPEC_ORIG       "RSPEC_ORIG"     // Original from reduction
#define RSPEC_PRE        "RSPEC_PRE"      // Prepared by coaddition
#define RSPEC            "RSPEC"          // Standard
#define RSPEC_SMOOTH     "RSPEC_SMOOTH"   // Smoothed during line detection
#define RSPEC_CONT       "RSPEC_CONT"     // Continuum-fitted
#define RSPEC_REM        "RSPEC_REM"      // With lines removed
#define RSPEC_GUESS      "RSPEC_GUESS"    // With Voigt-guessed lines
#define RSPEC_VOIGT      "RSPEC_VOIGT"    // With Voigt-fitted  lines
//#define RSPEC_SPLINE     "RSPEC_SPLINE"

// Instrument profile for rebinned spectra (RPROF)
#define RPROF            "RPROF"
#define RPROF_MULTIMR    "RPROF_MULTIMR"

// Masks (MASK)
#define MASK             "MASK"

// Lists of lines (LINE)
#define LINE             "LINE"
#define LINE_EQWIDTH     "LINE_EQWIDTH"
#define DET_LINE         "DET_LINE"
#define DET_LINE_NORM    "DET_LINE_NORM"
#define NOID_LINE        "NOID_LINE"

// List of free lines (FLINE)
#define FLINE_PRE        "FLINE_PRE"      // Prepared by line detection
#define FLINE_IDEN       "FLINE_IDEN"     // Identified
#define FLINE_REJ        "FLINE_REJ"      // Rejected by line identification
#define FLINE_REM        "FLINE_REM"      // Removed from spectrum
#define FLINE_GUESS      "FLINE_GUESS"    // With guess  Voigt parameters
#define FLINE_VOIGT      "FLINE_VOIGT"    // With fitted Voigt parameters

// Lists of lines in systems (SLINE)
#define SLINE_IDEN       "SLINE_IDEN"     // Identified
#define SLINE_GUESS      "SLINE_GUESS"    // With guess  Voigt parameters
#define SLINE_VOIGT      "SLINE_VOIGT"    // With fitted Voigt parameters


// Table with values of the Voigt function
#define VOIGT_FUNC       "VOIGT_FUNC"

// Lists of atomic transitions
#define QSO_ION          "QSO_ION"
#define QSO_ION_RED      "QSO_ION_RED"
#define STAR_ION         "STAR_ION"

// List of spectroscopic calibrations
#define LINE_RATIO_CALIB "LINE_RATIO_CALIB"
#define LINE_FEH_CALIB   "LINE_FEH_CALIB"

// Table with parameters
#define TEFF "TEFF"
#define FEH  "FEH"

/*
 * Others
 */

#define PRO_CATG         "ESO PRO CATG"

#define Z_EM             "ESO OCS OBJ Z EM"
#define RV               "ESO QC CCF RV"
#define QC_BERV          "ESO QC BERV"
#define QC_ORDER102_SNR  "ESO QC ORDER102 SNR"

#define QC_MASK_NROW     "ESO QC MASK NROW"

#define QC_COLDEN_THRES  "ESO QC COLDEN THRES"
#define QC_CONT_RCHISQ   "ESO QC CONT RCHISQ"
#define QC_DET_LINE_NUM  "ESO QC DET LINE NUM"
#define QC_ID_LINE_NUM   "ESO QC ID LINE NUM"
#define QC_LINE_RCHISQ   "ESO QC LINE RCHISQ"
#define QC_PIX_SIZE      "ESO QC PIX SIZE"

#define DAS_VERSION      "ESO DAS VERSION"
#define DAS_DATE         "ESO DAS DATE"

#define DAS_RV           "ESO DAS RV"
#define DAS_RV_MEAN      "ESO DAS RV MEAN"
#define DAS_SNR600_RMS   "ESO DAS SNR600 RMS"
#define DAS_VEL_STEP     "ESO DAS VEL STEP"
#define DAS_Z_EM         "ESO DAS Z_EM"

#define DAS_TEFF         "ESO DAS TEFF"
#define DAS_STDTEFF      "ESO DAS TEFF ERROR"
#define DAS_NCALTEFF     "ESO DAS TEFF NCALIB"
#define DAS_NCALINDTEFF  "ESO DAS TEFF NCALIBIND"
#define DAS_FEH          "ESO DAS FEH"
#define DAS_STDFEH       "ESO DAS FEH ERROR"
#define DAS_NCALFEH      "ESO DAS FEH NCALIB"


/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <string.h>
#include <math.h>

#include "espda_print.h"
#include "espda_util.h"

/*-----------------------------------------------------------------------------
                                Functions prototypes
 -----------------------------------------------------------------------------*/

const char     *espda_dfs_catg   (const cpl_propertylist *);

cpl_error_code  espda_dfs_groups (cpl_frameset           *);

const char     *espda_dfs_license(void);
cpl_error_code espda_files_dont_exist(cpl_frameset *frameset);
#endif
