/* $Id: espda_spec.h 252491 2020-04-01 16:47:36Z gcupani $
 *
 * This file is part of the ESPDA Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * $Author: gcupani $
 * $Date: 2020-04-01 16:47:36 +0000 (Wed, 01 Apr 2020) $
 * $Revision: 252491 $
 * $Name:  $
 */

#ifndef ESPDA_SPEC_H
#define ESPDA_SPEC_H

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define espda_spec_basic(A)                                      \
            espda_struct_initcol(A, "WAVEL",   CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "MASK",    CPL_TYPE_INT);    \
            espda_struct_initcol(A, "PIXSIZE", CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "FLUX",    CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "FLUXERR", CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "QUALDATA", CPL_TYPE_INT);   \

#define espda_spec_drstodas(A)                                   \
            if (air_flag) {					 \
            cpl_table_name_column(A, "wavelength_air", "WAVEL"); \
            cpl_table_erase_column(A, "wavelength");		 \
	    }						         \
            else {						 \
            cpl_table_name_column(A, "wavelength", "WAVEL");     \
            cpl_table_erase_column(A, "wavelength_air");         \
            }							 \
            if (cpl_table_has_column(A, "flux_cal")) {           \
            cpl_table_name_column(A, "flux_cal",   "FLUX");      \
            cpl_table_name_column(A, "error_cal",  "FLUXERR");   \
	    } \
	    else { \
            cpl_table_name_column(A, "flux",   "FLUX");      \
            cpl_table_name_column(A, "error",  "FLUXERR");   \
	    }	   \
            cpl_table_name_column(A, "quality", "QUALDATA"); \


#define espda_spec_fit(A)                                        \
            espda_spec_full(A);                                  \
            espda_spec_rms(A);                                   \
            espda_struct_initcol(A, "FIT",     CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "CONT",    CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "NORM",    CPL_TYPE_DOUBLE); \
            espda_struct_initcol(A, "NORMERR", CPL_TYPE_DOUBLE); \

#define espda_spec_full(A)                                      \
            espda_spec_basic(A);                                \
            espda_struct_initcol(A, "SPECID", CPL_TYPE_INT);    \
            espda_struct_initcol(A, "ORDID",  CPL_TYPE_INT);    \
            espda_struct_initcol(A, "EQFACT", CPL_TYPE_DOUBLE); \

#define espda_spec_rms(A)                                        \
            espda_spec_basic(A);                                 \
            espda_struct_initcol(A, "FLUXRMS", CPL_TYPE_DOUBLE); \


/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>
#include <math.h>
#ifdef _OPENMP
#  include <omp.h>
#endif
#include <sys/time.h>
#include <stdint.h>

#include "espda_dfs.h"
#include "espda_print.h"
#include "espda_struct.h"
//#include "sgfilter.h"


/*-----------------------------------------------------------------------------
                                   Functions prototypes
 -----------------------------------------------------------------------------*/

cpl_error_code  espda_spec_ave  (const cpl_table *,
                                 const char      *,
                                 const char      *,
                                 const char      *,
                                 const cpl_size,
                                 const cpl_size,
                                 double          *,
                                 double          *,
                                 double          *);

cpl_error_code espda_spec_clean (cpl_table       *,
                                 cpl_table       *,
                                 const char      *,
                                 const char      *,
                                 int,
                                 cpl_size        *);

cpl_array      *espda_spec_conv  (cpl_table       *,
                                  cpl_array       *,
                                  cpl_array       *);

cpl_error_code  espda_spec_corr  (cpl_table       *,
                                  double,
                                  double,
                                  int,
                                  cpl_table      **);

cpl_table      *espda_spec_equal (cpl_table 	 *,
                                  int,
                                  double,
                                  double);

cpl_error_code  espda_spec_fill   (cpl_frameset *,
                                     cpl_size,
                                     cpl_size,
                                     char         *,
                                     cpl_table    *);

cpl_table      *espda_spec_rebin (cpl_table       *,
                                  double,
                                  double,
                                  double,
                                  double);

cpl_table      *espda_spec_remove(cpl_table   *,
                                  double);

/*
cpl_error_code espda_spec_sg     (cpl_table  *,
                                 char       *,
                                 cpl_table  *,
                                 cpl_table  *);
*/
cpl_error_code espda_spec_shift (cpl_table       *,
                                 cpl_array       *);


#endif
